import unittest

import numpy as np

from conf.testing import monozstackmask
from extensions.chaeo.accessors import MonoPatchStack, MonoPatchStackFromFile, PatchStack3D



class TestCziImageFileAccess(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def test_make_patch_stack_from_3d_array(self):
        w = 256
        h = 512
        n = 4
        acc = MonoPatchStack(np.random.rand(h, w, n))
        self.assertEqual(acc.count, n)
        self.assertEqual(acc.hw, (h, w))
        self.assertEqual(acc.make_tczyx().shape, (n, 1, 1, h, w))

    def test_make_patch_stack_from_list(self):
        w = 256
        h = 512
        n = 4
        acc = MonoPatchStack([np.random.rand(h, w) for _ in range(0, n)])
        self.assertEqual(acc.count, n)
        self.assertEqual(acc.hw, (h, w))
        self.assertEqual(acc.make_tczyx().shape, (n, 1, 1, h, w))

    def test_make_patch_stack_from_file(self):
        h = monozstackmask['h']
        w = monozstackmask['w']
        c = monozstackmask['c']
        n = monozstackmask['z']
        acc = MonoPatchStackFromFile(monozstackmask['path'])
        self.assertEqual(acc.hw, (h, w))
        self.assertEqual(acc.count, n)
        self.assertEqual(acc.make_tczyx().shape, (n, c, 1, h, w))
        self.assertEqual(acc.fpath, monozstackmask['path'])

    def test_raises_filenotfound(self):
        from extensions.chaeo.accessors import FileNotFoundError
        with self.assertRaises(FileNotFoundError):
            acc = MonoPatchStackFromFile('c:/fake/file/name.tif')

    def test_patch_as_yxcz_array(self):
        w = 256
        h = 512
        n = 4
        acc = MonoPatchStack([np.random.rand(h, w) for _ in range(0, 4)])
        self.assertEqual(acc.iat_yxcz(0).shape, (h, w, 1, 1))

    def test_make_3d_patch_stack_from_list(self):
        w = 256
        h = 512
        nz = 5
        n = 4
        acc = PatchStack3D([np.random.rand(h, w, nz) for _ in range(0, n)])
        self.assertEqual(acc.count, n)
        self.assertEqual(acc.hw, (h, w))
        self.assertEqual(acc.chroma, 1)
        self.assertEqual(acc.iat(0).shape, (h, w, nz))

    def test_3d_patch_as_yxcz_array(self):
        w = 256
        h = 512
        nz = 5
        n = 4
        acc = PatchStack3D([np.random.rand(h, w, nz) for _ in range(0, n)])
        self.assertEqual(acc.iat_yxcz(0).shape, (h, w, 1, nz))